==> libtunepimp-0.2.0.tar.gz: libtunepimp-0.2.0/lib/filecache.cpp <==
/*----------------------------------------------------------------------------

   libtunepimp -- The MusicBrainz tagging library.  
                  Let a thousand taggers bloom!
   
   Copyright (C) Robert Kaye 2003
   
   This file is part of libtunepimp.

   libtunepimp is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   libtunepimp is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with libtunepimp; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

   $Id: filecache.cpp,v 1.6 2003/06/23 09:27:19 robert Exp $

----------------------------------------------------------------------------*/
#ifdef WIN32
#if _MSC_VER == 1200
#pragma warning(disable:4786)
#endif
#endif

#include "filecache.h"

FileCache::FileCache(void)
{
    numUnsaved = 0;
    serialNum = 0;
}

FileCache::~FileCache(void)
{
}

int FileCache::add(const string &fileName)
{
    map<int, pair<Track *, int> >::iterator  i;
    Track                                   *track = NULL;
    string                                   temp;
    int                                      ret;
    pair<Track *, int>                       p;

    mutex.acquire();
    for(i = cache.begin(); i != cache.end(); i++)
    {
        (*i).second.first->getFileName(temp);
        if (temp == fileName)
        {
            ret = (*i).first;
            mutex.release();
            return ret;
        }
    }

    track = new Track();
    track->setStatus(ePending);
    track->setFileName(fileName);

    p.first = track;
    p.second = 0;
    cache[serialNum] = p;
    xref[track] = serialNum++;

    mutex.release();

    return serialNum - 1;
}

void FileCache::remove(int index)
{
    map<int, pair<Track *, int> >::iterator  i;

    mutex.acquire();

    i = cache.find(index);
    if (i != cache.end())
    {
        if ((*i).second.second == 0)
            cache.erase(i);
        else
        {
            Track *track;

            track = (*i).second.first;
            track->lock();
            track->setStatus(eDeleted);
            track->unlock();
        }
    }
    mutex.release();
}

void FileCache::getFileIds(vector<int> &ids)
{
    map<int, pair<Track *, int> >::iterator  i;

    ids.clear();
    mutex.acquire();
    for(i = cache.begin(); i != cache.end(); i++)
        ids.push_back((*i).first);

    mutex.release();
}

int FileCache::getFileIdFromTrack(Track *track)
{
    map<Track *, int>::iterator j;
    int                         fileId = -1;

    mutex.acquire();

    j = xref.find(track);
    if (j != xref.end())
        fileId = (*j).second;

    mutex.release();

    return fileId;
}

int FileCache::getNumItems(void)
{
    int count;

    mutex.acquire();
    count = cache.size();
    mutex.release();

    return count;
}

int FileCache::getNumUnsavedItems(void)
{
    map<int, pair<Track *, int> >::iterator  i;
    int                                      count = 0;

    mutex.acquire();
    for(i = cache.begin(); i != cache.end(); i++)
        if ((*i).second.first->getStatus() == eRecognized)
        {
            if ((*i).second.first->hasChanged())
                count++;
        }
    mutex.release();

    return count;
}

void FileCache::getCounts(map<FileStatus, int> &counts)
{
    map<int, pair<Track *, int> >::iterator  i;

    mutex.acquire();
    for(i = cache.begin(); i != cache.end(); i++)
        counts[(*i).second.first->getStatus()]++;
    mutex.release();
}

void FileCache::getTracksFromStatus(FileStatus status, vector<Track *> &tracks)
{
    map<int, pair<Track *, int> >::iterator  i;

    tracks.clear();
    mutex.acquire();
    for(i = cache.begin(); i != cache.end(); i++)
        if ((*i).second.first->getStatus() == status)
        {
            (*i).second.second++;
            tracks.push_back((*i).second.first);
        }
    mutex.release();
}

Track *FileCache::getNextItem(FileStatus status)
{
    map<int, pair<Track *, int> >::iterator  i;
    Track                                   *track = NULL;

    mutex.acquire();
    for(i = cache.begin(); i != cache.end(); i++)
        if ((*i).second.first->getStatus() == status)
        {
            (*i).second.second++;
            track = (*i).second.first;
            break;
        }
    mutex.release();

    return track;
}

Track *FileCache::getTrackFromFileName(const string &fileName)
{
    map<int, pair<Track *, int> >::iterator  i;
    Track                                   *track = NULL;
    string                                   temp;

    mutex.acquire();
    for(i = cache.begin(); i != cache.end(); i++)
    {
        (*i).second.first->getFileName(temp);
        if (temp == fileName)
        {
            (*i).second.second++;
            track = (*i).second.first;
            break;
        }
    }
    mutex.release();

    return track;
}

Track *FileCache::getTrackFromTrackId(const string &trackId)
{
    map<int, pair<Track *, int> >::iterator  i;
    Track                                   *track = NULL;
    Metadata                                 data;

    mutex.acquire();
    for(i = cache.begin(); i != cache.end(); i++)
    {
        (*i).second.first->getServerMetadata(data);
        if (data.trackId == trackId)
        {
            (*i).second.second++;
            track = (*i).second.first;
            break;
        }
    }
    mutex.release();

    return track;
}

Track *FileCache::getTrack(int fileId)
{
    map<int, pair<Track *, int> >::iterator  i;
    Track                       *track = NULL;

    mutex.acquire();

    i = cache.find(fileId);
    if (i != cache.end())
    {
        (*i).second.second++;
        track = (*i).second.first;
    }

    mutex.release();

    return track;
}

void FileCache::release(Track *track)
{
    map<int, pair<Track *, int> >::iterator  i;
    map<Track *, int>::iterator              j;
    FileStatus                               status;
    int                                      index;

    mutex.acquire();

    j = xref.find(track);
    if (j != xref.end())
    {
        index = (*j).second;

        i = cache.find(index);
        if (i != cache.end())
        {
            (*i).second.second--;
            if ((*i).second.second == 0)
            {
                track->lock();
                status = track->getStatus();
                track->unlock();

                if (status == eDeleted)
                {
                    cache.erase(i);
                    xref.erase(j);        
                }
            }
        }
    }

    mutex.release();
}

==> libtunepimp-0.2.1.tar.gz: libtunepimp-0.2.1/lib/filecache.cpp <==
/*----------------------------------------------------------------------------

   libtunepimp -- The MusicBrainz tagging library.  
                  Let a thousand taggers bloom!
   
   Copyright (C) Robert Kaye 2003
   
   This file is part of libtunepimp.

   libtunepimp is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   libtunepimp is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with libtunepimp; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

   $Id: filecache.cpp,v 1.7 2003/07/03 08:52:49 robert Exp $

----------------------------------------------------------------------------*/
#ifdef WIN32
#if _MSC_VER == 1200
#pragma warning(disable:4786)
#endif
#endif

#include "filecache.h"

FileCache::FileCache(void)
{
    numUnsaved = 0;
    serialNum = 0;
}

FileCache::~FileCache(void)
{
}

int FileCache::add(const string &fileName)
{
    map<int, pair<Track *, int> >::iterator  i;
    Track                                   *track = NULL;
    string                                   temp;
    int                                      ret;
    pair<Track *, int>                       p;

    mutex.acquire();
    for(i = cache.begin(); i != cache.end(); i++)
    {
        (*i).second.first->getFileName(temp);
        if (temp == fileName)
        {
            ret = (*i).first;
            mutex.release();
            return ret;
        }
    }

    track = new Track();
    track->setStatus(ePending);
    track->setFileName(fileName);

    p.first = track;
    p.second = 0;
    cache[serialNum] = p;
    xref[track] = serialNum++;

    mutex.release();

    return serialNum - 1;
}

void FileCache::remove(int index)
{
    map<int, pair<Track *, int> >::iterator  i;

    mutex.acquire();

    i = cache.find(index);
    if (i != cache.end())
    {
        if ((*i).second.second == 0)
            cache.erase(i);
        else
        {
            Track *track;

            track = (*i).second.first;
            track->lock();
            track->setStatus(eDeleted);
            track->unlock();
        }
    }
    mutex.release();
}

void FileCache::getFileIds(vector<int> &ids)
{
    map<int, pair<Track *, int> >::iterator  i;

    ids.clear();
    mutex.acquire();
    for(i = cache.begin(); i != cache.end(); i++)
        ids.push_back((*i).first);

    mutex.release();
}

int FileCache::getFileIdFromTrack(Track *track)
{
    map<Track *, int>::iterator j;
    int                         fileId = -1;

    mutex.acquire();

    j = xref.find(track);
    if (j != xref.end())
        fileId = (*j).second;

    mutex.release();

    return fileId;
}

int FileCache::getNumItems(void)
{
    int count;

    mutex.acquire();
    count = cache.size();
    mutex.release();

    return count;
}

int FileCache::getNumUnsavedItems(void)
{
    map<int, pair<Track *, int> >::iterator  i;
    int                                      count = 0;

    mutex.acquire();
    for(i = cache.begin(); i != cache.end(); i++)
        if ((*i).second.first->getStatus() == eRecognized)
        {
            if ((*i).second.first->hasChanged())
                count++;
        }
    mutex.release();

    return count;
}

void FileCache::getCounts(map<TPFileStatus, int> &counts)
{
    map<int, pair<Track *, int> >::iterator  i;

    mutex.acquire();
    for(i = cache.begin(); i != cache.end(); i++)
        counts[(*i).second.first->getStatus()]++;
    mutex.release();
}

void FileCache::getTracksFromStatus(TPFileStatus status, vector<Track *> &tracks)
{
    map<int, pair<Track *, int> >::iterator  i;

    tracks.clear();
    mutex.acquire();
    for(i = cache.begin(); i != cache.end(); i++)
        if ((*i).second.first->getStatus() == status)
        {
            (*i).second.second++;
            tracks.push_back((*i).second.first);
        }
    mutex.release();
}

Track *FileCache::getNextItem(TPFileStatus status)
{
    map<int, pair<Track *, int> >::iterator  i;
    Track                                   *track = NULL;

    mutex.acquire();
    for(i = cache.begin(); i != cache.end(); i++)
        if ((*i).second.first->getStatus() == status)
        {
            (*i).second.second++;
            track = (*i).second.first;
            break;
        }
    mutex.release();

    return track;
}

Track *FileCache::getTrackFromFileName(const string &fileName)
{
    map<int, pair<Track *, int> >::iterator  i;
    Track                                   *track = NULL;
    string                                   temp;

    mutex.acquire();
    for(i = cache.begin(); i != cache.end(); i++)
    {
        (*i).second.first->getFileName(temp);
        if (temp == fileName)
        {
            (*i).second.second++;
            track = (*i).second.first;
            break;
        }
    }
    mutex.release();

    return track;
}

Track *FileCache::getTrackFromTrackId(const string &trackId)
{
    map<int, pair<Track *, int> >::iterator  i;
    Track                                   *track = NULL;
    Metadata                                 data;

    mutex.acquire();
    for(i = cache.begin(); i != cache.end(); i++)
    {
        (*i).second.first->getServerMetadata(data);
        if (data.trackId == trackId)
        {
            (*i).second.second++;
            track = (*i).second.first;
            break;
        }
    }
    mutex.release();

    return track;
}

Track *FileCache::getTrack(int fileId)
{
    map<int, pair<Track *, int> >::iterator  i;
    Track                       *track = NULL;

    mutex.acquire();

    i = cache.find(fileId);
    if (i != cache.end())
    {
        (*i).second.second++;
        track = (*i).second.first;
    }

    mutex.release();

    return track;
}

void FileCache::release(Track *track)
{
    map<int, pair<Track *, int> >::iterator  i;
    map<Track *, int>::iterator              j;
    TPFileStatus                               status;
    int                                      index;

    mutex.acquire();

    j = xref.find(track);
    if (j != xref.end())
    {
        index = (*j).second;

        i = cache.find(index);
        if (i != cache.end())
        {
            (*i).second.second--;
            if ((*i).second.second == 0)
            {
                track->lock();
                status = track->getStatus();
                track->unlock();

                if (status == eDeleted)
                {
                    cache.erase(i);
                    xref.erase(j);        
                }
            }
        }
    }

    mutex.release();
}
